from __future__ import print_function, unicode_literals
from ..lib.handler.youtrack_handler import YoutrackHandler
from ..lib.handler.github_handler import GithubHandler
from ..lib.handler import git_handler as git
from ..lib.config import Config
from ..lib.logger import Logger
from ..lib.handler import prompt_utils

import re

youtrack = YoutrackHandler()
github = GithubHandler()
logger = Logger()
config = Config().load()


def entrypoint(args):
    git.is_dirty(args.project)

    if args.card:
        issue = youtrack.get_issue(args.card)
    else:
        slug = ask_project_slug()
        issue = select_card(slug)

    checkout_branch(args.project, issue)

    youtrack.assign_to(issue["id"], "me")

    youtrack.update_state(issue["id"], config["youtrack"]["picked_state"])


def ask_project_slug():
    return prompt_utils.ask_questions_input(
        'Inserisci slug progetto Youtrack: ', config["youtrack"]["default_project_prefix"])


def ask_board():
    boards = youtrack.get_boards()
    choices = [{"name": i["name"], "value": i["name"]} for i in boards]
    return prompt_utils.ask_choices(
        'Seleziona board: ', choices, config["youtrack"]["default_board_name"])


def select_card(slug):
    board = ask_board()
    issues = youtrack.get_issues(
        slug,
        "Board {}: {{Current Sprint}}".format(
            board
        ),
        0,
        100
    )
    choices = [{"name": issue_label(i), "value": i} for i in issues if i["State"] in ["ToDo", "In Progress"]]
    return prompt_utils.ask_choices(
        'Seleziona card: ', choices)


def issue_label(issue):
    return "{} {}".format(issue["id"], issue["summary"])


def checkout_branch(project, issue):
    branch_name = prompt_utils.ask_questions_input("Inserisci nome del branch: ", re.sub(
        r'([\s\\.~\^:\[\]"\'?]|[^\x00-\x7F])+', "-", issue["summary"]
    ).lower())

    parent_branch_name = prompt_utils.ask_questions_input(
        "Inserisci branch iniziale: ", "master")

    branch_type = issue["Type"].lower().replace(" ", "-")

    branch_name = "{}/{}/{}".format(
        branch_type, branch_name, issue["id"]
    )

    git.checkout(project, parent_branch_name)

    git.checkout(project, branch_name)



from __future__ import print_function, unicode_literals
from ..lib.handler.youtrack_handler import YoutrackHandler
from ..lib.handler.github_handler import GithubHandler
from ..lib.handler.slack_handler import SlackHandler
from ..lib.handler import prompt_utils
from ..lib.logger import Logger
from ..lib.config import Config
import sys
import semver

youtrack = YoutrackHandler()
github = GithubHandler()
config = Config().load()
logger = Logger()
slack = SlackHandler()


def entrypoint(args):
    if prompt_utils.ask_confirm("Sono presenti migration?", default=False):
        logger.error(
            "Impossibile continuare con il deploy a causa di migration presenti. Chiedi ai devops di effettuare il deploy.")
        sys.exit(-1)

    repo = github.get_repo(args.project)

    latest_release = get_release(repo)

    versions = bump_versions(latest_release.title)

    new_version = prompt_utils.ask_choices(
        "Seleziona versione:",
        [
            {"name": "Patch {}".format(versions["patch"]), "value": versions["patch"]},
            {"name": "Minor {}".format(versions["minor"]), "value": versions["minor"]},
            {"name": "Major {}".format(versions["major"]), "value": versions["major"]},
        ]
    )

    commits = github.get_commits_since_release(repo, latest_release)
    message = "\n".join(["* " + c.commit.message.splitlines()[0] for c in commits])

    logger.info("\nLista dei commit:\n{}\n".format(message))

    if not prompt_utils.ask_confirm("Vuoi continuare?"):
        sys.exit(-1)

    release_state = config["youtrack"]["release_state"]

    logger.info("Imposto le card in {}".format(release_state))

    deployed_cards_link = []
    try:
        for issue_id in youtrack.get_issue_ids(commits):
            deployed_cards_link.append(youtrack.get_link(issue_id))
            youtrack.update_state(
                issue_id, release_state
            )
    except:
        logger.warning("Si è verificato un errore durante lo spostamento delle card in {}".format(release_state))
    finally:
        create_release(repo, new_version, message, args.project, deployed_cards_link)


def get_release(repo):
    latest_release = github.get_latest_release_if_exists(repo)
    if latest_release:
        logger.info("La release attuale è {}".format(latest_release.title))
        return latest_release
    else:
        tag = repo.get_tags()[0]
        logger.info("L'ultimo tag trovato è {}".format(tag.name))

        return repo.create_git_release(tag.name, tag.name, "New release from tag {}".format(tag.name))


def create_release(repo, new_version, message, project, deployed_cards_link):
    new_release = repo.create_git_release(new_version, new_version, message)
    if new_release:
        logger.info("La release è stata creata! Link: {}".format(new_release.html_url))
        slack.post(
            "#deploy", "ho effettuato il deploy di {}. Nuova release con versione {} {}\n{}".format(project, new_version, new_release.html_url, deployed_cards_to_string(deployed_cards_link)))


def bump_versions(current):
    return {
        "patch": semver.bump_patch(current),
        "minor": semver.bump_minor(current),
        "major": semver.bump_major(current)
    }


def deployed_cards_to_string(cards):
    if len(cards) == 0:
        return ""
    else:
        return "\n".join(cards)



from __future__ import print_function, unicode_literals
from ..lib.handler.youtrack_handler import YoutrackHandler
from ..lib.handler.github_handler import GithubHandler
from ..lib.logger import Logger
from ..lib.qainit import qainit_deploy
from cement import shell

from ..lib.handler import git_handler as git  # import GitHandler
from ..lib.handler import drone_handler as drone
import json
import subprocess
import sys

youtrack = YoutrackHandler()
github = GithubHandler()
logger = Logger()


def entrypoint(args):
    pull_requests = select_pull_requests()
    branches = list(map(lambda pr: github.get_branch_from_pr(
        pr.project.name, pr.github_id), pull_requests))
    dict = {"projects": {}}

    for index, pr in enumerate(pull_requests):
        dict["projects"].update(
            {pr.project.name: {
                "branch": branches[index].name, "revision": branches[index].commit.sha[:14]}}
        )

    dump = json.dumps(dict)
    print(dump)

    json_arg = json.dumps(dump)
    print(json_arg)

    qainit_deploy(json_arg)

    youtrack.update_state(pull_requests[0].youtrack_id, "QA")
    for pr in pull_requests:
        db.update_pr_state(pr, "QA")

    logger.info("Creazione del qa lanciata! Puoi seguire lo stato su {}".format(
        drone.get_last_build_url('qainit')))


def select_pull_requests():
    options = db.get_all_owned_prs_with_qa(user, ['In Progress', 'Review'])

    if not options:
        logger.warning(
            "Non ci sono pull requests su progetti che hanno un qa, nulla da fare")
        sys.exit(0)
    else:
        p = shell.Prompt('Scegli la pull request da mandare in qa',
                         options=options,
                         numbered=True
                         )

    selection = p.prompt()
    other_prs = list(filter(lambda pr: pr.youtrack_id
                            == selection.youtrack_id and pr.project_id != selection.project_id, options))

    if other_prs:
        logger.info(
            "Ci sono altre pull requests con lo stesso youtrack id di quella selezionata:")
        for pr in other_prs:
            print(pr)

        p = shell.Prompt("Vuoi aggiungerle alla build del qa?",
                         options=['yes', 'no'])

        if (p.prompt() == 'yes'):
            return [selection] + other_prs

    return [selection]



from __future__ import print_function, unicode_literals
from .logger import Logger
from .config import Config

from .handler import git_handler as git
import subprocess
# import sys

config = Config().load()
logger = Logger()

# twig_command =
# '{}/twig-binaries/bin/twig-feature'.format(config['user']['projects_home'])
# development only
qainit_dir = '{}/qainit'.format(config['user']['projects_home'])


def qainit_deploy(args):
    git.check_repo_cloned('qainit')
    return subprocess.run(
        ['twig', 'feature', 'suite', 'deploy', args], cwd=qainit_dir, check=True)


def qainit_stop(branch):
    git.check_repo_cloned('qainit')
    git.sync('qainit')
    if git.remote_branch_exists('qainit', branch):
        try:
            subprocess.run(['twig', 'feature', 'suite', 'stop',
                            branch], cwd=qainit_dir, check=True)
        except BaseException:
            logger.error(
                "Non sono riuscito a spegnere il qa, per favore spegnilo manualmente!\nDevops are watching you! ( •͡˘ _•͡˘)")

    else:
        logger.warning(
            "Non sono riuscito a trovare un qa per questa issue, se il qa esiste, per favore spegnilo manualmente\nDevops are watching you! ( •͡˘ _•͡˘)")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
from ..lib.handler.captainhook_handler import CaptainHook
from ..lib.logger import Logger
import sys

logger = Logger()
lock = CaptainHook()


def entrypoint(args):
    if(args.action in ['lock', 'l']):
        req = lock.lock_project(args.project)
        handle_request(req)
        logger.info(
            'Bloccato deploy su staging del progetto {}'.format(args.project))
    elif(args.action in ['unlock', 'u']):
        req = lock.unlock_project(args.project)
        handle_request(req)
        logger.info(
            'Abilitato deploy su staging del progetto {}'.format(args.project))
    else:
        logger.warning('Non ho capito che cosa devo fare')
        sys.exit(-1)


def handle_request(request):
    if request.status_code == 401:
        logger.error("Impossibie contattare captainhook, stai usando la VPN?")
        sys.exit(-1)
    if request.status_code == 500:
        logger.error("Si è verificato un errore su captainhook. Chiedi ai devops di verificare (X_X)")
        sys.exit(-1)
    if request.status_code != 200:
        logger.error("Qualcosa è andato storto durante la richiesta.")
        sys.exit(-1)

    return True



#################### Maintained by Hatch ####################
# This file is auto-generated by hatch. If you'd like to customize this file
# please add your changes near the bottom marked for 'USER OVERRIDES'.
# EVERYTHING ELSE WILL BE OVERWRITTEN by hatch.
#############################################################
from io import open

from setuptools import find_packages, setup

with open('suite_py/__version__.py', 'r') as f:
    for line in f:
        if line.startswith('__version__'):
            version = line.strip().split('=')[1].strip(' \'"')
            break
    else:
        version = '0.0.1'

with open('README.rst', 'r', encoding='utf-8') as f:
    readme = f.read()

REQUIRES = [
    "prima-youtrack>=0.2",
    "PyGithub>=1.43",
    "cement>=3.0",
    "PyInquirer>=1.0.3",
    "PyYaml>=5.1",
    "slackclient>=2.1",
    "termcolor>=1.1.0",
    "autoupgrade-prima>=0.3",
    "semver>=2.9.0",
]

kwargs = {
    'name': 'suite-py',
    'version': version,
    'description': '',
    'long_description': readme,
    'author': 'Prima Assicurazioni S.p.A',
    'author_email': 'devops@prima.it',
    'maintainer': 'larrywax, EugenioLaghi, michelangelomo',
    'maintainer_email': 'devops@prima.it',
    'url': 'https://github.com/primait/suite_py',
    'license': 'MIT/Apache-2.0',
    'classifiers': [
        # 'Development Status :: 4 - Beta',
        # 'Intended Audience :: Developers',
        # 'License :: OSI Approved :: MIT License',
        # 'License :: OSI Approved :: Apache Software License',
        # 'Natural Language :: English',
        # 'Operating System :: OS Independent',
        # 'Programming Language :: Python :: 3.5',
        # 'Programming Language :: Python :: 3.6',
        # 'Programming Language :: Python :: Implementation :: CPython',
        # 'Programming Language :: Python :: Implementation :: PyPy',
    ],
    'install_requires': REQUIRES,
    'tests_require': ['coverage', 'pytest'],
    'packages': find_packages(exclude=('tests', 'tests.*')),
    'entry_points': {
        'console_scripts': [
            'suite-py = suite_py.__main__:main',
        ],
    },

}

#################### BEGIN USER OVERRIDES ####################
# Add your customizations in this section.

###################### END USER OVERRIDES ####################

setup(**kwargs)



from __future__ import print_function, unicode_literals
from ..tokens import Tokens
from ..config import Config
from ..singleton import Singleton
from youtrack.connection import Connection
from youtrack.youtrack import YouTrackException
from PyInquirer import prompt

import os
import re

tokens = Tokens()
config = Config().load()


class YoutrackHandler(metaclass=Singleton):
    _client = None

    def __init__(self):
        self._client = Connection(
            url=config['youtrack']['url'], token=tokens.youtrack)

    def get_users(self):
        return self._client.get_users()

    def get_issue(self, issue_id):
        return self._client.get_issue(issue_id)

    def get_issues(self, slug, filter, min, max):
        return self._client.get_issues(slug, filter, min, max)

    def get_boards(self):
        return self._client.get_all_agiles()

    def validate_issue(self, issue_id):
        try:
            if self.get_issue(issue_id):
                return True
        except (YouTrackException) as e:
            return False

    def comment(self, issue_id, comment):
        self._client.execute_command(issue_id, 'comment', comment=comment)

    def update_state(self, issue_id, status):
        self._client.execute_command(issue_id, 'State {}'.format(status))

    def add_tag(self, issue_id, label):
        self._client.execute_command(issue_id, 'tag {}'.format(label))

    def assign_to(self, issue_id, user):
        self._client.execute_command(issue_id, "Assignee {}".format(user))

    def select_issue(self):
        issue = prompt([
            {
                'type': 'input',
                'name': 'name',
                'message': "Inserisci il numero della issue su youtrack",
                'default': "PRIMA-000",
                'validate': lambda val: self.validate_issue(val)
            }
        ])

        return self.get_issue(issue['name'])

    def get_link(self, issue_id):
        return "https://prima-assicurazioni-spa.myjetbrains.com/youtrack/issue/{}".format(issue_id)

    def update_issue(self, issue_id, summary, description):
        return self._client.update_issue(issue_id, summary, description)

    def get_issue_ids(self, commits):
        issue_ids = []
        for c in commits:
            issue_id = self.get_card_from_name(c.commit.message)
            if issue_id:
                issue_ids.append(issue_id)
        return issue_ids

    def get_card_from_name(self, name):
        regex = '[A-Z]+-[0-9]+'
        if re.search(regex, name):
            id_card = re.findall(regex, name)[0]
            if not self.validate_issue(id_card):
                return None
            return id_card
        else:
            return None



from __future__ import print_function, unicode_literals
from ..tokens import Tokens
from ..config import Config
from ..singleton import Singleton
from PyInquirer import prompt
import slack

import os

tokens = Tokens()


class SlackHandler(metaclass=Singleton):
    _client = None

    def __init__(self):
        self._client = slack.WebClient(token=tokens.slack)

    def post(self, channel, text):
        self._client.chat_postMessage(
            channel=channel,
            text=text,
            as_user=True
        )



from __future__ import print_function, unicode_literals
from ..lib.handler.youtrack_handler import YoutrackHandler
from ..lib.handler.github_handler import GithubHandler
from ..lib.handler.captainhook_handler import CaptainHook
from ..lib.config import Config
from ..lib.logger import Logger
from ..lib.handler import prompt_utils
from ..lib.qainit import qainit_stop
from ..lib.handler import git_handler as git
from ..lib.handler import drone_handler as drone
import os
import sys
import re

youtrack = YoutrackHandler()
github = GithubHandler()
logger = Logger()
lock = CaptainHook()
config = Config().load()


def entrypoint(args):
    stop_if_master_locked(args.project)

    pr = select_pr(args.project)
    branch_name = pr.head.ref

    if not pr.mergeable or pr.mergeable_state != "clean":
        logger.error("Impossibile fare il merge. Controlla che le review siano positive o che la build non sia fallita. {}".format(pr.html_url))
        sys.exit(-1)

    logger.info("Eseguo il merge...")

    merge_status = pr.merge(
        commit_title="{} (#{})".format(pr.title, pr.number), commit_message='', merge_method='squash')

    if not merge_status.merged:
        logger.error("Si è verificato un errore durante il merge.")
        sys.exit(-1)

    drone_build = get_drone_build(args.project, merge_status.sha)

    if drone_build:
        logger.info(
            "Pull request mergiata su master! Puoi seguire lo stato della build su {}".format(drone_build))
    else:
        logger.info("Pull request mergiata su master!")

    logger.info("Cancello il branch da github...")

    if git.delete_remote_branch(args.project, branch_name):
        logger.info("Branch remoto eliminato.")
    else:
        logger.error("Si è verificato un errore durante la cancellazione del branch remoto.")
        logger.error("Elimina il branch manualmente da {}".format(pr.html_url))

    if prompt_utils.ask_confirm("Vuoi bloccare staging? (Necessario se bisogna testare su staging)".format(args.project), default=False):
        lock.lock_project(args.project)

    logger.info("Aggiorno lo stato della card su youtrack...")
    if update_card(branch_name):
        logger.info("Tutto fatto!")
    else:
        logger.warning("Non sono riuscito a trovare una issue YouTrack nel nome del branch o la issue indicata non esiste.\
            Nessuna card aggiornata su YouTrack")

    # logger.info("Spengo il qa, se esiste")
    # qainit_branch = "{}_feature/{}".format(
    #     git.get_git_username(), branch_name)
    # qainit_stop(qainit_branch)

    logger.info("Se esiste un qa per questo branch, per favore, spegnilo.")

    sys.exit()


def select_pr(project):
    prs = github.get_list_pr(project)

    choices = [{"name": "{} {}".format(i.number, i.title), "value": i} for i in prs]
    return prompt_utils.ask_choices('Seleziona PR: ', choices)


def stop_if_master_locked(project):
    request = lock.status(project)
    if request.status_code != 200:
        logger.error("Impossibile determinare lo stato del lock su master.")
        sys.exit(-1)

    if request.json()["locked"]:
        logger.error("Il progetto è lockato su master. Impossibile continuare.")
        sys.exit(-1)


def get_drone_build(project, sha):
    drone_build = drone.get_pr_build_url(
        project, sha)
    if drone_build != "":
        return drone_build
    return None


def update_card(branch_name):
    id_card = youtrack.get_card_from_name(branch_name)

    if id_card:
        youtrack.update_state(id_card, config["youtrack"]["merged_state"])
        return True
    else:
        return False
